//*****************************************************************************
//  SENSOR CONTROLLER STUDIO TASKS: IrRX AND IrTX WITH LCD SHOWING DATA (22.09.2016)
//  Operating system: TI-RTOS
//
//  The Sensor Controller is used to sample an IR signal using CompA triggered on the receive LED
//  and timer0 for the sampling period to count high and low time. The Sensor Controller wakes the CM3 to verify
//  whether a valid IR signal has been received (checks the command byte and its inverted byte).
//
//  Once the received data is valid the user can press one of the SMARTRF06EB buttons(right, left, down, up)to start the IrTX task
//  and the transmission is started by passing the data to the IrTX task.
//  The received data is saved on the pressed button and can be deleted upon pressing the boards SELECT button.
// 	**VERY IMPORTANT**: Due to the Rx/Tx pin programming, after a successful receive sequence a single transmission can be initiated.
//	However receiving IR data can be repeated many times.
//  Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
//
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//    Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//****************************************************************************/
/* XDCtools Header files */
#include <xdc/std.h>
#include <xdc/runtime/System.h>


/* TI-RTOS Header files */
#include <ti/mw/display/Display.h>
#include <ti/mw/display/DisplayExt.h>

#include <driverlib/ioc.h>


#include "ex_include_tirtos.h"
#include "scif.h"
#include "string.h"
#include "stdio.h"
#include "Board.h"



#define BV(n)               (1 << (n))
#define TASKSTACKSIZE   512


// Display error message if the SCIF driver has been generated with incorrect operating system setting
#ifndef SCIF_OSAL_TIRTOS_H
    #error "SCIF driver has incorrect operating system configuration for this example. Please change to 'TI-RTOS' in the Sensor Controller Studio project panel and re-generate the driver."
#endif

// Display error message if the SCIF driver has been generated with incorrect target chip package
#ifndef SCIF_TARGET_CHIP_PACKAGE_QFN48_7X7_RGZ
    #error "SCIF driver has incorrect target chip package configuration for this example. Please change to 'QFN48 7x7 RGZ' in the Sensor Controller Studio project panel and re-generate the driver."
#endif

// Task data
Task_Struct myTask,task1Struct;
Char myTaskStack[512];
Char task1Stack[TASKSTACKSIZE];

// Semaphore used to wait for Sensor Controller task ALERT event
static Semaphore_Struct semScTaskAlert;
// Semaphore used to wait for buttons to be pressed
static Semaphore_Struct semButtonPressed;
// Semaphore used to wait for lcd
static Semaphore_Struct sem1Struct;
Task_Handle lcdLowPriorityTask;

PIN_Config pLedPinTable[] = {
 	Board_LED3 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX, // Data transmission OK
    Board_LED4 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX, // received package OK
    PIN_TERMINATE
};
PIN_State ledPinState;
PIN_Handle hLedPins;


void scCtrlReadyCallback(void) {
} // scCtrlReadyCallback


void scTaskAlertCallback(void) {

    // Wake up the OS task
    Semaphore_post(Semaphore_handle(&semScTaskAlert));

} // scTaskAlertCallback

PIN_Config pButtonPinTable[] = {
	// IR-TX trigger
	Board_KEY_DOWN  | PIN_INPUT_EN  | PIN_PULLUP | PIN_HYSTERESIS,
	Board_KEY_UP  | PIN_INPUT_EN  | PIN_PULLUP | PIN_HYSTERESIS,
    Board_KEY_LEFT  | PIN_INPUT_EN  | PIN_PULLUP | PIN_HYSTERESIS,
    Board_KEY_RIGHT | PIN_INPUT_EN  | PIN_PULLUP | PIN_HYSTERESIS,
    Board_KEY_SELECT | PIN_INPUT_EN  | PIN_PULLUP | PIN_HYSTERESIS,
    PIN_TERMINATE
};
PIN_State buttonPinState;
PIN_Handle hButtonPins;


void buttonPressedCallback(PIN_Handle handle, PIN_Id pinId) {

    // Disable button pin interrupt
    PIN_setConfig(hButtonPins, PIN_BM_IRQ, Board_KEY_LEFT | PIN_IRQ_DIS);
    PIN_setConfig(hButtonPins, PIN_BM_IRQ, Board_KEY_RIGHT | PIN_IRQ_DIS);
    PIN_setConfig(hButtonPins, PIN_BM_IRQ, Board_KEY_UP | PIN_IRQ_DIS);
    PIN_setConfig(hButtonPins, PIN_BM_IRQ, Board_KEY_DOWN | PIN_IRQ_DIS);
    PIN_setConfig(hButtonPins, PIN_BM_IRQ, Board_KEY_SELECT | PIN_IRQ_DIS);

    // Resume the button OS task
    Semaphore_post(Semaphore_handle(&semButtonPressed));
} // buttonPressedCallback


unsigned int i = 0;
uint16_t bit_stream [40] = {};
uint32_t data = 0; //irCmdAdr = 0xFFFFFFFF;
uint32_t button,keydown,keyup,keyright,keyleft,left,right,down,up = 0;



void taskFxn(UArg a0, UArg a1) {
    // Initialize the Sensor Controller
    scifOsalInit();
    scifOsalRegisterCtrlReadyCallback(scCtrlReadyCallback);
    scifOsalRegisterTaskAlertCallback(scTaskAlertCallback);
    scifInit(&scifDriverSetup);
    //Add drive strength for 8mA on DIO6!!!!
    IOCIODrvStrengthSet(IOID_6, IOC_CURRENT_8MA, PIN_DRVSTR_MAX);

    scifStartRtcTicksNow(0x3333);//200ms run time interval
    //Start IR_RX task TO MEASURE SIGNAL PULSES LENGTH and decode it to 32-bit data
    scifStartTasksNbl(BV(SCIF_RXIRLED_TASK_ID));

    // Main loop
	while (1) {
		PIN_setOutputValue(hLedPins, Board_LED3, 0);
    	// Wait for an ALERT callback
    	Semaphore_pend(Semaphore_handle(&semScTaskAlert), BIOS_WAIT_FOREVER);
    	// Clear the ALERT interrupt source
    	scifClearAlertIntSource();

    	unsigned short address = scifTaskData.rxirled.output.address ;
    	unsigned short command = scifTaskData.rxirled.output.command ;
    	unsigned char command_low 	= (command>>8)& 0xFF;			//extract the data from the buffer
    	unsigned char command_high 	= (command)& 0xFF;

    	//Check for valid signal()
    	if((command_low ^ command_high)== 0xff){   //Command Byte is repeated with the Bits inverted: check if this is the case!
    		data = (address<<16) | command;		//Address and Command in IR NEC Protocol
            //SHOW ON LED (only on SRF06EB) THAT A VALID SIGNAL WAS RECEIVED
    		PIN_setOutputValue(hLedPins, Board_LED4, 1);
    		Task_sleep(100 * (5000/Clock_tickPeriod));
    		PIN_setOutputValue(hLedPins, Board_LED4, 0);
    		scifAckAlertEvents();
		}else{
			scifAckAlertEvents();
		}
        scifResetTaskStructs(BV(SCIF_RXIRLED_TASK_ID), 0);
	}
} // taskFxn




Void lcdLowPriorityFxn(UArg a0, UArg a1) {
    // Enable button pins, with interrupt handler buttonPressedCallback()
    hButtonPins = PIN_open(&buttonPinState, pButtonPinTable);
    PIN_registerIntCb(hButtonPins, buttonPressedCallback);
    //LCD CONFIG
    Display_Params params;
	Display_Params_init(&params);
	params.lineClearMode = DISPLAY_CLEAR_BOTH;
	Display_Handle hDisplayLcd = Display_open(Display_Type_LCD, &params);
	/* Check if the selected Display type was found and successfully opened */
	if (hDisplayLcd) {
		Display_print0(hDisplayLcd, 0, 3, "NEC IR protocol");
		/* Wait a while so text can be viewed. */
		Task_sleep(1000 * (1000/Clock_tickPeriod));
		/* Not all displays have a GraphicsLib back-end */
		Display_print0(hDisplayLcd, 1, 1, "AFTER RECEIVING");
		Display_print0(hDisplayLcd, 2, 1, "PLEASE PRESS");
		Display_print0(hDisplayLcd, 3, 1, "RIGHT BUTTON!");
		/* Wait for a bit, then clear */
		Task_sleep(5000 * (1000/Clock_tickPeriod));
		Display_clear(hDisplayLcd);
	}

    while (1) {
    	//Set Interrupt on Board RIGHT button
		PIN_setConfig(hButtonPins, PIN_BM_IRQ, Board_KEY_RIGHT | PIN_IRQ_NEGEDGE);
		PIN_setConfig(hButtonPins, PIN_BM_IRQ, Board_KEY_LEFT | PIN_IRQ_NEGEDGE);
		PIN_setConfig(hButtonPins, PIN_BM_IRQ, Board_KEY_DOWN | PIN_IRQ_NEGEDGE);
		PIN_setConfig(hButtonPins, PIN_BM_IRQ, Board_KEY_UP | PIN_IRQ_NEGEDGE);
		PIN_setConfig(hButtonPins, PIN_BM_IRQ, Board_KEY_SELECT | PIN_IRQ_NEGEDGE);
		Semaphore_pend(Semaphore_handle(&semButtonPressed), BIOS_WAIT_FOREVER);
        PIN_setOutputValue(hLedPins, Board_LED3, 1);
        //Save the data upon pressing on one of the buttons left right down up.
        if (PIN_getInputValue(Board_KEY_LEFT) == 0) {
        	if(left==0){
				keyleft=data;
				button=keyleft;
				left=1;
			}else{
				button=keyleft;
			}
        } else if (PIN_getInputValue(Board_KEY_RIGHT) == 0) {
        	if(right==0){
				keyright=data;
				button=keyright;
				right=1;
			}else{
				button=keyright;
			}
        } else if (PIN_getInputValue(Board_KEY_DOWN) == 0){
        	if(down==0){
				keydown=data;
				button=keydown;
				down=1;
        	}else{
        		button=keydown;
        	}
        } else if (PIN_getInputValue(Board_KEY_UP) == 0){
        	if(up==0){
				keyup=data;
				button=keyup;
				up=1;
			}else{
				button=keyup;
			}
        }else if(PIN_getInputValue(Board_KEY_SELECT) == 0){
        	//Initialise all buttons
        	up = 0;
        	down= 0;
        	right= 0;
        	left=0;
        }
        data = 0;
		scifTaskData.rxirled.input.enable=0;
		//DISPLAY DATA ON LCD
        Display_print0(hDisplayLcd, 0, 2, "START TRANSMISSION");
		Display_print1(hDisplayLcd, 1, 2, "IR DATA 0x0%x", button); //button
		//Handover the 4*Bytes Data to the IR-TX task.
		scifTaskData.rxirled.input.address = (button>>16) & 0xffff;
		scifTaskData.rxirled.input.command = button & 0xffff;
        scifResetTaskStructs(BV(SCIF_RXIRLED_TASK_ID), 0);
		scifExecuteTasksOnceNbl(BV(SCIF_RXIRLED_TASK_ID));
    }
}

int main(void) {
    // Initialize the PIN driver
    //PIN_init(BoardGpioInitTable);
	Task_Params taskParams;
    Semaphore_Params semParams;

    /* Call board init functions */
    Board_initGeneral();
    // Configure the OS task
    Task_Params_init(&taskParams);
    taskParams.stack = myTaskStack;
    taskParams.stackSize = sizeof(myTaskStack);
    taskParams.priority = 3;
    Task_construct(&myTask, taskFxn, &taskParams, NULL);

	/* Construct lcd Task threads */
 	taskParams.stack = &task1Stack;
	taskParams.stackSize = sizeof(task1Stack);
	taskParams.priority = 3;
	Task_construct(&task1Struct, lcdLowPriorityFxn, &taskParams, NULL);

	// Create Semaphore used to wait for CPU Alert of the RX Task
    Semaphore_Params_init(&semParams);
    semParams.mode = Semaphore_Mode_BINARY;
    Semaphore_construct(&semScTaskAlert, 0, &semParams);

    // Create the semaphore used to wait button pressing
    Semaphore_Params_init(&semParams);
    semParams.mode = Semaphore_Mode_BINARY;
    Semaphore_construct(&semButtonPressed, 0, &semParams);

    BIOS_start();
    return 0;
	} // main
