/*
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/***** Includes *****/
#include <stdlib.h>
#include <xdc/std.h>
#include <xdc/runtime/System.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>

/* Drivers */
#include <ti/drivers/rf/RF.h>
#include <ti/drivers/PIN.h>

/* Board Header files */
#include "Board.h"
#include <ti/sysbios/knl/Clock.h> //FG 25.10.16 PWR

#include "smartrf_settings/smartrf_settings.h"

//MS: 7.3.16
#include "sce/scif_framework.h"
#include "sce/scif_osal_tirtos.h"
#include "sce/scif.h"

#define BV(n)               (1 << (n))

#define DISABLE_LOW_POWER   0

#include <ti/drivers/UART.h>
#include <ti/drivers/uart/UARTCC26XX.h>
#include "string.h"
#include "stdio.h"
#include <driverlib/aux_adc.h> //for ADC calibration operations

#include "wmbus/mbus_packet.h"

Clock_Struct clkStruct; //FG 25.10.16 PWR
Void clkFxn(UArg arg0); //FG 25.10.16 PWR
Clock_Handle clkHandle; //FG 25.10.16 PWR

// UART driver objects
UART_Handle uHandle;
UART_Params uParams;
char pLine[21], command = '0';

// Semaphore used to wait for Sensor Controller task ALERT event
static Semaphore_Struct semScTaskAlert, semStruct; //Semaphores for SCE and LCD
Semaphore_Handle semHandle, semLCDHandle;

// Task data
#define TASKSTACKSIZE   512
Task_Struct myTask, task0Struct;
Char myTaskStack[TASKSTACKSIZE * 2], task0Stack[TASKSTACKSIZE];


/* LCD definitions */                                       // 01/26/2016
uint8_t LCD_stat = 0;                       				// LCD state variable for COM signal generation
static PIN_Handle lcdCOMPinHandle;                          // 01/26/2016
static PIN_State lcdCOMPinState;                                                // 01/26/2016

static PIN_Handle lcdSegPinHandle;                                              // 01/27/2016
static PIN_State lcdSegPinState;                                                // 01/27/2016


/******************************************************************************
* Application LCD pin configuration table
*
* S2         DIO_15
* S4         DIO_6
* S6         DIO_21
* S8         DIO_20
* S9         DIO_19
* S10        DIO_18
* S11        DIO_17
* S12        DIO_14
* S13        DIO_13
* S14        DIO_12
* S15        DIO_11
* S16        DIO_10
* No S17 = DIO_9, used for INV replacement
* S18        DIO_8
* S20        DIO_7
* S22        DIO_5
* S24        DIO_4
* S27        DIO_1
*
* BASE_PWM_N DIO_9
* Base_PWM   DIO_27
* COM1_1     DIO_22
* COM2_1     DIO_23
* COM3_1     DIO_24
* COM4_1     DIO_25
*
* last modification: March 17, 2016
*
* 03/03/2016 added inverted Base_PWM
*
* 01/27/2016 adding definitions for segment drive
* 01/26/2016 implementation
******************************************************************************/
PIN_Config lcdCOMPinTableState0[] = {
  Base_PWM_N | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX, // 03/03/2016
  Base_PWM | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
  COM1_1 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
  COM2_1 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_HIGH | PIN_PUSHPULL | PIN_DRVSTR_MAX,
  COM3_1 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_HIGH | PIN_PUSHPULL | PIN_DRVSTR_MAX,
  COM4_1 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_HIGH | PIN_PUSHPULL | PIN_DRVSTR_MAX,
  PIN_TERMINATE                                                                 // 01/26/2016
};

PIN_Config lcdCOMPinTableStateOff[] = {
  Base_PWM_N | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX, // 03/03/2016
  Base_PWM | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
  COM1_1 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
  COM2_1 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
  COM3_1 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
  COM4_1 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
  PIN_TERMINATE                                                                 // 01/26/2016
};


//uint32_t lcdCOMState0 = 0x03800000;         // COM Port State for phase 0       // 01/27/2016
uint32_t lcdCOMState0 = 0x03800200;         // COM Port State for phase 0       // 03/03/2016
uint32_t lcdSegXCOM1ActState0 = 0x003FFFFE; // Seg Pins Active state for        // 01/28/2016
                                            // phase 0
uint32_t lcdSegXCOM1NActState0 = 0x00000000;// Seg OutputEnable state for       // 01/28/2016
                                            // phase 0


uint32_t lcdCOMState2 = 0x04400000;         // COM Port State for phase 2       // 01/27/2016
uint32_t lcdSegXCOM1ActState2 = 0x00000000; // Seg Pins Active state for        // 01/28/2016
                                            // phase 2
uint32_t lcdSegXCOM1NActState2 = 0x00000000;// Seg OutputEnable state for       // 01/28/2016
                                            // phase 2

//uint32_t lcdCOMState4 = 0x03400000;         // COM Port State for phase 4       // 01/27/2016
uint32_t lcdCOMState4 = 0x03400200;         // COM Port State for phase 4       // 03/17/2016
uint32_t lcdSegXCOM2ActState4 = 0x003FFFFE; // Seg Pins Active state for        // 01/28/2016
                                            // phase 4
uint32_t lcdSegXCOM2NActState4 = 0x00000000;// Seg OutputEnable state for       // 01/28/2016
                                            // phase 4

uint32_t lcdCOMState6 = 0x04800000;         // COM Port State for phase 6       // 01/27/2016
uint32_t lcdSegXCOM2ActState6 = 0x00000000; // Seg Pins Active state for        // 01/28/2016
                                            // phase 6
uint32_t lcdSegXCOM2NActState6 = 0x00000000;// Seg OutputEnable state for       // 01/28/2016
                                            // phase 6

//uint32_t lcdCOMState8 = 0x02C00000;         // COM Port State for phase 8       // 01/27/2016
uint32_t lcdCOMState8 = 0x02C00200;         // COM Port State for phase 8       // 03/17/2016
uint32_t lcdSegXCOM3ActState8 = 0x003FFFFE; // Seg Pins Active state for        // 01/28/2016
                                            // phase 8
uint32_t lcdSegXCOM3NActState8 = 0x00000000;// Seg OutputEnable state for       // 01/28/2016
                                            // phase 8

uint32_t lcdCOMState10 = 0x05000000;        // COM Port State for phase 10      // 01/27/2016
uint32_t lcdSegXCOM3ActState10 = 0x00000000;// Seg Pins Active state for        // 01/28/2016
                                            // phase 10
uint32_t lcdSegXCOM3NActState10 = 0x00000000;// Seg OutputEnable state for      // 01/28/2016
                                            // phase 10

//uint32_t lcdCOMState12 = 0x01C00000;        // COM Port State for phase 12      // 01/27/2016
uint32_t lcdCOMState12 = 0x01C00200;        // COM Port State for phase 12      // 03/17/2016
uint32_t lcdSegXCOM4ActState12 = 0x003FFFFE;// Seg Pins Active state for        // 01/28/2016
                                            // phase 12
uint32_t lcdSegXCOM4NActState12 = 0x00000000;// Seg OutputEnable state for      // 01/28/2016
                                            // phase 12

uint32_t lcdCOMState14 = 0x06000000;        // COM Port State for phase 14      // 01/27/2016
uint32_t lcdSegXCOM4ActState14 = 0x00000000;// Seg Pins Active state for        // 01/28/2016
                                            // phase 14
uint32_t lcdSegXCOM4NActState14 = 0x00000000;// Seg OutputEnable state for      // 01/28/2016
                                            // phase 14

PIN_Config lcdSegPinTableState[] = {
  LCD_S0 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S2 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S4 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S6 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S8 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S9 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S10 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S11 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S12 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S13 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S14 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S15 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S16 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
//  LCD_S17 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,	03/17/2016
  LCD_S18 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S20 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S22 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S24 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S25 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S26 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  LCD_S27 | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_DIS | PIN_PUSHPULL,
  PIN_TERMINATE                                                                 // 01/26/2016
};

uint16_t LCDFrameCntr = 0;                  // LCD Frame Counter defining the   // 01/28/2016
                                            // LCD update rate with the Segment
                                            // activation Demo
//uint8_t LCDSegCntr = 0;                     // LCD Segment Counter, used to   // 01/29/2016  // 01/28/2016
//uint8_t LCDSegCntr = 1;                     // display all                      // 01/29/2016
uint8_t LCDSegCntr = 2;                     // display CC1310                   // 01/29/2016
//uint8_t LCDSegCntr = 3;                     // display HCA                      // 01/29/2016
//uint8_t LCDSegCntr = 4;                     // display HCA, 3, 4, 5 digit       // 01/31/2016
                                            // LCD Segment Counter, used to
                                            // activate one segment after
                                            // another in the segment
                                            // activation Demo


// Semaphore used to wait for PCB Cap Touch task event
static Semaphore_Struct semScTaskAlert;

// Semaphore used to wait for ADC sampler ALERT processing to complete
static Semaphore_Struct semLCDOn;

void scCtrlReadyCallback(void);
void scTaskAlertCallback(void);

void scCtrlReadyCallback(void) {

} // scCtrlReadyCallback


void scTaskAlertCallback(void) {

    // Wake up the OS task
    Semaphore_post(Semaphore_handle(&semScTaskAlert));

} // scTaskAlertCallback

//MS 1. Oct'15
//MS

extern uint8_t TXpacket[];
extern uint8_t TXbytes[];
extern uint8_t TXdataLength;
extern uint16_t  packetLength, TmodeByteLength;

//ADC calculation objects
uint32_t calADC12_gain=0, lmt70_HeatSensor=0, lmt70_RoomSensor=0, lmt70_CalInput=0;
int8_t calADC12_offset;
float temp1_LMT70lin,temp2_LMT70lin, temp1_LMT70dual,temp2_LMT70dual;
float a = -7.857923E-06, b = -1.777501E-01, c = 2.046398E+02;
uint32_t packetCounter=0, pcbTouchCounter=0, finishCount = 0;

/*******************************************************************************
*               Packet Examples from EN13757-4:2014
*   Packet S2:  0F 44 AE 0C 78 56 34 12 01 07 44 47 78 0B 13 43 65 87 1E 6D     (20 Bytes) S- and T-Modes with Format A
*   Packet C2:  14 44 AE 0C 78 56 34 12 01 07 8C 20 27 78 0B 13 43 65 87 7A C5  (21 Bytes) all other Modes with Format B
*/

uint8_t wMBusPacketC[]= {0x14,PACKET_C_FIELD, 0xAE, 0x0C, PACKET_CI_FIELD, 0x56, 0x34, 0x12, MAN_ID, MAN_VER, 0x8C, 0x20, 0x27, PACKET_CI_FIELD, 0x0B, 0x13, 0x43, 0x65, 0x87, 0x00, 0x00/*0x7A, 0xC5*/}; //C2-Mode
//volatile unsigned int  mode = 3; //C-mode
//volatile unsigned int  mode = 2; //T-mode
volatile unsigned int  mode = 0; //S-mode
extern uint16_t  packetLength, TmodeByteLength;
extern uint8_t TXbytes[];


/* Pin driver handle */
static PIN_Handle ledPinHandle;
/*
 * Application LED pin configuration table:
 *   - All LEDs board LEDs are off.
 */
PIN_Config pinTable[] =
{
    Board_LED0 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
	Board_LED1 | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
    PIN_TERMINATE
};


/***** Defines *****/
#define TX_TASK_STACK_SIZE 1024
#define TX_TASK_PRIORITY   2

/* TX Configuration */
#define PAYLOAD_LENGTH      120	//was 30,; 80 and 120 Bytes work with SRF7 Studi0
#define PACKET_INTERVAL     (uint32_t)(4000000*30.0f) /* Set packet interval to 30 secs (*30.0f) */


/***** Prototypes *****/
static void txTaskFunction(UArg arg0, UArg arg1);

/***** Variable declarations *****/
static Task_Params txTaskParams;
Task_Struct txTask;    /* not static so you can see in ROV */
static uint8_t txTaskStack[TX_TASK_STACK_SIZE];

static RF_Object rfObject;
static RF_Handle rfHandle;

uint32_t time;
static uint8_t packet[PAYLOAD_LENGTH];
//static uint16_t seqNumber;
static PIN_Handle pinHandle;


/***** Function definitions *****/



/******************************************************************************
*  ======== heartBeatFxn ========
*  Toggle the Board_LED0. The Task_sleep is determined by arg0 which
*  is configured for the heartBeat Task instance.
*
*  last modification: January 29th 2016
*
*  01/29/2016 adding segment tests
*  01/28/2016 adding Segment functionality
*  01/27/2016 debugging COM functionality
*  01/26/2016 adding LCD COM GPIO functionality
******************************************************************************/
Void heartBeatFxn(UArg arg0, UArg arg1)
{
    while (1) {
    	// SW Project defines Tick every 10us in rfPacketTx.cfg
        Task_sleep(400); //MS: 12.3.16 Sleep for 100 * 10us Tick = 1ms, FG: changed to 4ms
        if (finishCount == 0)
          Semaphore_pend(semLCDHandle, BIOS_WAIT_FOREVER);

        finishCount++;

        if (finishCount == 500) {	//MS 14.3.16: 2000 = 2 secs; 4000 = 4secs

			PIN_setPortOutputValue(lcdCOMPinHandle,  0x00000000);
												  // push COM + Base_PWM values into  // 01/27/2016
												  // port output
			PIN_setPortOutputValue(lcdSegPinHandle, 0x00000000);
												  // push Segment Pin output values   // 01/28/2016
												  // into port output
			PIN_setPortOutputEnable(lcdSegPinHandle, 0xFFFFFFFF);
												  // switch Output buffers on/off     // 01/28/2016
												  // according to desired Segment activity
			finishCount = 0; //reset count
			//Toggle between "ALL 84 or 80 segs ON" and "CC1310" 22.2.2016
			if (LCDSegCntr > 1)
			  LCDSegCntr = 1;
			else
			  LCDSegCntr += 1;

			LCD_stat = 0;
			LCDFrameCntr = 0;
			//Semaphore_post(semLCDHandle); //FG: Enable if TxTask_init(ledPinHandle) is disabled.

        }

        else { //finishCount is < 2000 or 4000

          switch(LCD_stat)                                                        // 01/26/2016
          {
          case 0:                             // 1st phase P1
            LCD_stat = 2;                     // go to next state
            //lcdCOMPinHandle = PIN_open(&lcdCOMPinState, lcdCOMPinTableState0);
                                              // allocate LCD COM Pins to LCD     // 01/27/2016 // 01/26/2016
                                              //  function
            PIN_setPortOutputValue(lcdCOMPinHandle, lcdCOMState0);
                                              // push COM + Base_PWM values into  // 01/27/2016
                                              // port output
            PIN_setPortOutputEnable(lcdSegPinHandle, lcdSegXCOM1NActState0);
                                              // switch Output buffers on/off     // 01/28/2016
                                              // according to desired Segment
                                              // activity
            PIN_setPortOutputValue(lcdSegPinHandle, lcdSegXCOM1ActState0);
                                              // push Segment Pin output values   // 01/28/2016
                                              // into port output

            break;
          case 2:                             // 2nd phase P2
            LCD_stat = 4;                     // go to next state
            //lcdCOMPinHandle = PIN_open(&lcdCOMPinState, lcdCOMPinTableState2);
                                              // allocate LCD COM Pins to LCD     // 01/26/2016
                                              //  function
            PIN_setPortOutputValue(lcdCOMPinHandle, lcdCOMState2);
                                              // push COM + Base_PWM values into  // 01/27/2016
                                              // port output
            PIN_setPortOutputEnable(lcdSegPinHandle, lcdSegXCOM1NActState2);
                                              // switch Output buffers on/off     // 01/28/2016
                                              // according to desired Segment
                                              // activity
            PIN_setPortOutputValue(lcdSegPinHandle, lcdSegXCOM1ActState2);
                                              // push Segment Pin output values   // 01/28/2016
                                              // into port output

            break;
          case 4:                             // 3rd phase P3
            LCD_stat = 6;                     // go to next state
            //lcdCOMPinHandle = PIN_open(&lcdCOMPinState, lcdCOMPinTableState4);
                                              // allocate LCD COM Pins to LCD     // 01/26/2016
                                              //  function
            PIN_setPortOutputValue(lcdCOMPinHandle, lcdCOMState4);
                                              // push COM + Base_PWM values into  // 01/27/2016
                                              // port output
            PIN_setPortOutputValue(lcdSegPinHandle, lcdSegXCOM2ActState4);
                                              // push Segment Pin output values   // 01/28/2016
                                              // into port output
            PIN_setPortOutputEnable(lcdSegPinHandle, lcdSegXCOM2NActState4);
                                              // switch Output buffers on/off     // 01/28/2016
                                              // according to desired Segment
                                              // activity
            break;
          case 6:                             // 4th phase P4
            LCD_stat = 8;                     // go to next state
            //lcdCOMPinHandle = PIN_open(&lcdCOMPinState, lcdCOMPinTableState6);
                                              // allocate LCD COM Pins to LCD     // 01/26/2016
                                              //  function
            PIN_setPortOutputValue(lcdCOMPinHandle, lcdCOMState6);
                                              // push COM + Base_PWM values into  // 01/27/2016
                                              // port output
            PIN_setPortOutputValue(lcdSegPinHandle, lcdSegXCOM2ActState6);
                                              // push Segment Pin output values   // 01/28/2016
                                              // into port output
            PIN_setPortOutputEnable(lcdSegPinHandle, lcdSegXCOM2NActState6);
                                              // switch Output buffers on/off     // 01/28/2016
                                              // according to desired Segment
                                              // activity
            break;
          case 8:                             // 5th phase P5
            LCD_stat = 10;                    // go to next state
            //lcdCOMPinHandle = PIN_open(&lcdCOMPinState, lcdCOMPinTableState8);
                                              // allocate LCD COM Pins to LCD     // 01/26/2016
                                              //  function
            PIN_setPortOutputValue(lcdCOMPinHandle, lcdCOMState8);
                                              // push COM + Base_PWM values into  // 01/27/2016
                                              // port output
            PIN_setPortOutputValue(lcdSegPinHandle, lcdSegXCOM3ActState8);
                                              // push Segment Pin output values   // 01/28/2016
                                              // into port output
            PIN_setPortOutputEnable(lcdSegPinHandle, lcdSegXCOM3NActState8);
                                              // switch Output buffers on/off     // 01/28/2016
                                              // according to desired Segment
                                              // activity
            break;
          case 10:                            // 6th phase P6
            LCD_stat = 12;                    // go to next state
            //lcdCOMPinHandle = PIN_open(&lcdCOMPinState, lcdCOMPinTableState10);
                                              // allocate LCD COM Pins to LCD     // 01/26/2016
                                              //  function
            PIN_setPortOutputValue(lcdCOMPinHandle, lcdCOMState10);
                                              // push COM + Base_PWM values into  // 01/27/2016
                                              // port output
            PIN_setPortOutputValue(lcdSegPinHandle, lcdSegXCOM3ActState10);
                                              // push Segment Pin output values   // 01/28/2016
                                              // into port output
            PIN_setPortOutputEnable(lcdSegPinHandle, lcdSegXCOM3NActState10);
                                              // switch Output buffers on/off     // 01/28/2016
                                              // according to desired Segment
                                              // activity
            break;
          case 12:                            // 7th phase P7
            LCD_stat = 14;                    // go to next state
            //lcdCOMPinHandle = PIN_open(&lcdCOMPinState, lcdCOMPinTableState12);
                                              // allocate LCD COM Pins to LCD     // 01/26/2016
                                              //  function
            PIN_setPortOutputValue(lcdCOMPinHandle, lcdCOMState12);
                                              // push COM + Base_PWM values into  // 01/27/2016
                                              // port output
            PIN_setPortOutputValue(lcdSegPinHandle, lcdSegXCOM4ActState12);
                                              // push Segment Pin output values   // 01/28/2016
                                              // into port output
            PIN_setPortOutputEnable(lcdSegPinHandle, lcdSegXCOM4NActState12);
                                              // switch Output buffers on/off     // 01/28/2016
                                              // according to desired Segment
                                              // activity
            break;
          case 14:                            // 8th phase P8
            LCD_stat = 0;                     // go to next state
            //lcdCOMPinHandle = PIN_open(&lcdCOMPinState, lcdCOMPinTableState14);
                                              // allocate LCD COM Pins to LCD     // 01/26/2016
                                              //  function
            PIN_setPortOutputValue(lcdCOMPinHandle, lcdCOMState14);
                                              // push COM + Base_PWM values into  // 01/27/2016
                                              // port output
            PIN_setPortOutputValue(lcdSegPinHandle, lcdSegXCOM4ActState14);
                                              // push Segment Pin output values   // 01/28/2016
                                              // into port output
            PIN_setPortOutputEnable(lcdSegPinHandle, lcdSegXCOM4NActState14);
                                              // switch Output buffers on/off     // 01/28/2016
                                              // according to desired Segment
                                              // activity
            break;
          default:
            break;
          }
          LCDFrameCntr++;                     // increment Frame counter          // 01/28/2016
                                              // defines LCD update rate
          if (LCDFrameCntr == 150)                                                // 01/28/2016
          {
            /* Segment activation Demo - activating one segment after another */  // 01/28/2016
            //LCDSegCntr++;                     // increment
            switch (LCDSegCntr)
            {
            case 1:
              //lcdSegXCOM1NActState0 = 0x00010000;

                                              // activate Segment 1A              // 01/28/2016
              //lcdSegXCOM1NActState2 = 0x00010000;

                                              // activate Segment 1A              // 01/28/2016
              //lcdSegXCOM2NActState4 |= 0x00010000;

                                              // activate Segment 1B              // 01/29/2016
              //lcdSegXCOM2NActState6 |= 0x00010000;

                                              // activate Segment 1B              // 01/29/2016
              /* Display all connected LCD segments */
              lcdSegXCOM1NActState0 = 0x003FFFFE;                                 // 01/29/2016
              lcdSegXCOM1NActState2 = 0x003FFFFE;                                 // 01/29/2016
              lcdSegXCOM2NActState4 = 0x003FFFFE;                                 // 01/29/2016
              lcdSegXCOM2NActState6 = 0x003FFFFE;                                 // 01/29/2016
              lcdSegXCOM3NActState8 = 0x003FFFFE;                                 // 01/29/2016
              lcdSegXCOM3NActState10 = 0x003FFFFE;                                // 01/29/2016
              lcdSegXCOM4NActState12 = 0x003FFFFE;                                // 01/29/2016
              lcdSegXCOM4NActState14 = 0x003FFFFE;                                // 01/29/2016
              break;                                                              // 01/28/2016
            case 2:
              /* Display CC1310 */                                                // 01/29/2016
              lcdSegXCOM1NActState0 = 0x00344414;                                 // 01/29/2016
              lcdSegXCOM1NActState2 = 0x00344414;                                 // 01/29/2016
              lcdSegXCOM2NActState4 = 0x000005D0;                                 // 01/29/2016
              lcdSegXCOM2NActState6 = 0x000005D0;                                 // 01/29/2016
              lcdSegXCOM3NActState8 = 0x002404D4;                                 // 01/29/2016
              lcdSegXCOM3NActState10 = 0x002404D4;                                // 01/29/2016
              lcdSegXCOM4NActState12 = 0x00240104;                                // 01/29/2016
              lcdSegXCOM4NActState14 = 0x00240104;                                // 01/29/2016
              //LCDSegCntr = 0;
              break;                                                              // 01/28/2016
            case 3:                                                               // 01/29/2016
              /* Display HCA, digit 2, 3, 4 */                                    // 01/29/2016
              lcdSegXCOM1NActState0 = 0x00341080;                                 // 01/29/2016
              lcdSegXCOM1NActState2 = 0x00341080;                                 // 01/29/2016
              lcdSegXCOM2NActState4 = 0x00205080;                                 // 01/29/2016
              lcdSegXCOM2NActState6 = 0x00205080;                                 // 01/29/2016
              lcdSegXCOM3NActState8 = 0x00245080;                                 // 01/29/2016
              lcdSegXCOM3NActState10 = 0x00245080;                                // 01/29/2016
              lcdSegXCOM4NActState12 = 0x00040000;                                // 01/29/2016
              lcdSegXCOM4NActState14 = 0x00040000;                                // 01/29/2016
              break;
            case 4:                                                               // 01/31/2016
              /* Display HCA, digit 3, 4, 5 */                                    // 01/31/2016
              lcdSegXCOM1NActState0 = 0x00041580;                                 // 01/31/2016
              lcdSegXCOM1NActState2 = 0x00041580;                                 // 01/31/2016
              lcdSegXCOM2NActState4 = 0x00140500;                                 // 01/31/2016
              lcdSegXCOM2NActState6 = 0x00140500;                                 // 01/31/2016
              lcdSegXCOM3NActState8 = 0x00141500;                                 // 01/31/2016
              lcdSegXCOM3NActState10 = 0x00141500;                                // 01/31/2016
              lcdSegXCOM4NActState12 = 0x00001000;                                // 01/31/2016
              lcdSegXCOM4NActState14 = 0x00001000;                                // 01/31/2016
              break;

            }
            LCDFrameCntr = 0;                 // reset Frame Counter              // 01/28/2016
          }
      }
  }
}

void TxTask_init(PIN_Handle inPinHandle)
{
    pinHandle = inPinHandle;

    Task_Params_init(&txTaskParams);
    txTaskParams.stackSize = TX_TASK_STACK_SIZE;
    txTaskParams.priority = TX_TASK_PRIORITY;
    txTaskParams.stack = &txTaskStack;
    txTaskParams.arg0 = (UInt)1000000;

    Task_construct(&txTask, txTaskFunction, &txTaskParams, NULL);
}

static void txTaskFunction(UArg arg0, UArg arg1)
{
    uint32_t time;
    RF_Params rfParams;
    RF_Params_init(&rfParams);

    RF_cmdPropTx.pktLen = PAYLOAD_LENGTH;
    RF_cmdPropTx.pPkt = packet;
    RF_cmdPropTx.startTrigger.triggerType = TRIG_ABSTIME;
    RF_cmdPropTx.startTrigger.pastTrig = 0; //FG: Packets are sent with an regular interval
    RF_cmdPropTx.startTime = 0;

    /* Request access to the radio */
    rfHandle = RF_open(&rfObject, &RF_prop, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup, &rfParams);

    /* Set the frequency */
    RF_postCmd(rfHandle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal, NULL, 0);

    /* Get current time */
    time = RF_getCurrentTime();

    /* defined in mbus_packet.h */
    mode = WMbus_mode;

    while(1)
    {
        /* 1. Create wM-Bus packet for each Mode: S, T or C */

    	uint8_t i;

    	if (mode == WMBUS_CMODE || mode == WMBUS_C2MODE || mode == WMBUS_NabefMODE || WMbus_mode == WMBUS_NcdMODE || WMbus_mode == WMBUS_NgMODE ||
    						    		 	 WMbus_mode == WMBUS_FMODE ) {
    		wMBusPacketC[16] = calADC12_offset; //(unsigned char) ((ui16wmbusACC_Counter&0x00FF0000)>>16);
    		wMBusPacketC[17] = calADC12_gain >> 8; //(unsigned char) ((ui16wmbusACC_Counter&0x0000FF00)>>8);
    		//wMBusPacketC[18] = calADC12_gain; //(unsigned char) (ui16wmbusACC_Counter&0x000000FF);
            wMBusPacketC[18] = packetCounter++; //(unsigned char) (ui16wmbusACC_Counter&0x000000FF);
    	    /* MS 2. Oct'15: use LMT70 data
    	            packet[2] = calADC12_offset;
    	            packet[3] = calADC12_gain >> 24;
    	            packet[4] = calADC12_gain >> 16;
    	            packet[5] = calADC12_gain >> 8;
    	            packet[6] = calADC12_gain;
    	    */
    				packetLength = packetSize (wMBusPacketC[0]);
    				encodeTXPacket(TXbytes, wMBusPacketC, packetLength);
    	            for (i = 0; i < packetLength; i++)  {
    	              packet[i] = TXbytes[i];               //wMBusPacketC[i-2];  //original: random data
    	               //MS 2. Oct'15: use LMT70 data
    	            }
    	}
    	else if (mode == WMBUS_TMODE) { //0x04 1st Byte??
    				TXpacket[13] = DIF_FIELD; //0x0B; need overwrite due to TXpacket->TXbytes->TXpacket handling
    				TXpacket[14] = VIF_FIELD; //0x13;
    				encodeTXPacket(TXbytes, TXpacket, TXdataLength);

    				//calculate T-mode TXpacket length
    				packetLength = packetSize (TXbytes[0]); //works for S-mode packet with Manchester in HW
    				encodeTXBytesTmode(TXpacket, TXbytes, packetLength);
    				TmodeByteLength = byteSize(2, 1, packetLength);
    				for (i = 0; i < TmodeByteLength; i++)  {
    	                          packet[i] = TXpacket[i];               //wMBusPacketC[i-2];  //original: random data
    	                          //MS 2. Oct'15: use LMT70 data
    				}
    			//	cc112xSpiWriteTxFifo(TXpacket, TmodeByteLength);
    	}
    	else if (mode == WMBUS_SMODE) {
    			    encodeTXPacket(TXbytes, TXpacket, TXdataLength);
    			    packetLength = packetSize (TXpacket[0]); //works for S-mode packet with Manchester in HW
    				//cc112xSpiWriteTxFifo(TXbytes, packetLength);
    	                    for (i = 0; i < packetLength; i++)  {
    	                          packet[i] = TXbytes[i];               //wMBusPacketC[i-2];  //original: random data
    	                          //MS 2. Oct'15: use LMT70 data
    	                    }
    	}


        /* 2. Set absolute TX time to utilize automatic power management */
        time += PACKET_INTERVAL;
        RF_cmdPropTx.startTime = time;

        /* 3. Send packet */
        RF_EventMask result = RF_runCmd(rfHandle, (RF_Op*)&RF_cmdPropTx, RF_PriorityNormal, NULL, 0);
        if (!(result & RF_EventLastCmdDone))
        {
            /* Error */
            while(1);
        }

        PIN_setOutputValue(pinHandle, Board_LED1,!PIN_getOutputValue(Board_LED1));
    }
}

void taskFxn(UArg a0, UArg a1) {

    // Initialize the Sensor Controller
    scifOsalInit();
    scifOsalRegisterCtrlReadyCallback(scCtrlReadyCallback);
    scifOsalRegisterTaskAlertCallback(scTaskAlertCallback);
    scifInit(&scifDriverSetup);

    // Set the Sensor Controller task tick interval to 1 second (was 1/16 second)
    scifStartRtcTicksNow(0x00010000 * 1); //was /16

    // Start the Sensor Controller tasks (not to be confused with OS tasks)
    // have to start BOTH tasks at the same time using | operator; 10.3.2016!!!
    scifStartTasksNbl(BV(SCIF_CAP_TOUCH_BUTTON_TASK_ID) | BV(SCIF_TRXIRLED_TASK_ID)); //| BV(SCIF_LMT70_TASK_ID 17.10.16
    //scifStartTasksNbl(BV(SCIF_CAP_TOUCH_BUTTON_TASK_ID)); //| BV(SCIF_LMT70_TASK_ID 17.10.16 //FG

    //Read out ADC calibration values for Analog Output from LMT70 to CC1310 ADC
    calADC12_offset = AUXADCGetAdjustmentOffset(AUXADC_REF_FIXED);
    calADC12_gain = AUXADCGetAdjustmentGain(AUXADC_REF_FIXED);

    // Main loop
    while (1) {
    	//

    	// Wait for an ALERT callback
    	Semaphore_pend(semHandle, BIOS_WAIT_FOREVER);
        // Clear the ALERT interrupt source
        scifClearAlertIntSource();	//7.3.2016 Interrupt when PCB touched by finger
        // Get the alert events
        uint32_t bvAlertEvents = scifGetAlertEvents();

        if (bvAlertEvents & BV(SCIF_CAP_TOUCH_BUTTON_TASK_ID))
        {
			pcbTouchCounter++;	 //MS: 10.2.16 CapTouch event is detected reliably every 1sec when PCB touched
			//CapTouch event detected, activate LCD display
			//pass Semaphore to HeartBeatFxn()
			Semaphore_post(semLCDHandle);
        }

        if (bvAlertEvents & BV(SCIF_LMT70_TASK_ID))
        {
            clkHandle = Clock_handle(&clkStruct);
            Clock_stop(clkHandle);

			#ifdef SCIF_FLOW_METER_DIO_OP_DIFF
			// Print TDC values
	 //       sprintf(pLine + 0,  "%4d ",  scifTaskData.lmt70powerRead.output.pTdcValues[0]);
	 //       sprintf(pLine + 5,  "%4d\n", scifTaskData.lmt70powerRead.output.pTdcValues[1]);
			#else
				lmt70_HeatSensor = AUXADCAdjustValueForGainAndOffset(scifTaskData.lmt70.output.adcVal1, calADC12_gain, calADC12_offset);
				lmt70_RoomSensor = AUXADCAdjustValueForGainAndOffset(scifTaskData.lmt70.output.adcVal2, calADC12_gain, calADC12_offset);
				//Linear Interpolation method for LMT70 temp. calculation
				// Thus the final equation used to calculate the measured temperature (TM) in the range between 20C and 30C is: TM = m  VTAO + b
				// TM = 0.193 C/mV  VTAO + 212.009C
				// where VTAO is in mV and TM is in C.
				temp1_LMT70lin = -0.193 * lmt70_HeatSensor * 1440 / 4095 + 212.009;
				temp2_LMT70lin = -0.193 * lmt70_RoomSensor * 1440 / 4095 + 212.009;
				temp1_LMT70dual = a * lmt70_HeatSensor * lmt70_HeatSensor * 1440*1440/(4095*4095) + b * lmt70_HeatSensor * 1440 / 4095 + c;
				temp2_LMT70dual = a * lmt70_RoomSensor * lmt70_RoomSensor * 1440*1440/(4095*4095)+  b * lmt70_RoomSensor * 1440 / 4095 + c;
			#endif
			Semaphore_post(semHandle);
        }
        // Acknowledge the alert event
        scifAckAlertEvents();

    }

}

/*
 *  ======== main ========
 */
int main(void)
{
	/* Call board init functions. */
    Board_initGeneral();
    Task_Params taskParams;

	//FG 25.10.16
    //One immidiate M3 wake up event to allow recharge period algorithm to find an optimal recharge period
    /* Construct BIOS Objects*/
    Clock_Params clkParams;
    Clock_Params_init(&clkParams);
    clkParams.period = 500000/Clock_tickPeriod;
    clkParams.startFlag = TRUE;
    /* Construct a one-shot Clock Instance*/
	Clock_construct(&clkStruct, (Clock_FuncPtr)clkFxn, 1500000/Clock_tickPeriod, &clkParams);

	// MS 7.3.16
    //Task_Params taskParams;

    // Optional: Prevent the system from entering standby to allow debugging
#if DISABLE_LOW_POWER
    Power_setConstraint(Power_SB_DISALLOW);
    Power_setConstraint(Power_IDLE_PD_DISALLOW);
#endif


    // MS: 7.3.16
    // Configure the OS task
    Task_Params_init(&taskParams);
    taskParams.stack = myTaskStack;
    taskParams.arg0 = 1000000  / Clock_tickPeriod;// 1ms                            // 14.3.16
    taskParams.stackSize = sizeof(myTaskStack);
    taskParams.priority = 2;
    Task_construct(&myTask, taskFxn, &taskParams, NULL);

    /* Construct heartBeat Task for LCD GPIO control */
    //MS 10.3.2016
     Task_Params_init(&taskParams);
     taskParams.arg0 = 1000000  / Clock_tickPeriod;// 1ms                            // 01/25/2016
     taskParams.stackSize = TASKSTACKSIZE;
     taskParams.stack = &task0Stack;
     taskParams.priority = 3;
     Task_construct(&task0Struct, (Task_FuncPtr)heartBeatFxn, &taskParams, NULL);


     //MS 10.3.2016
    // Create the semaphore used to wait for Sensor Controller ALERT events
    Semaphore_Params semParams;
    Semaphore_Params_init(&semParams);
    semParams.mode = Semaphore_Mode_BINARY;
    Semaphore_construct(&semScTaskAlert, 0, &semParams);
    // MS: 10.3.16
    semHandle = Semaphore_handle(&semScTaskAlert);

    // Create the semaphore used to wait for ADC sampling of both LMT70A sensors
    //MS 10.3.16
    Semaphore_construct(&semLCDOn, 0, &semParams);
    semLCDHandle = Semaphore_handle(&semLCDOn);
    //MS 10.3.16

    /* Open LCD COM pins */                                                     // 01/26/2016

    lcdCOMPinHandle = PIN_open(&lcdCOMPinState, lcdCOMPinTableState0);
                                              // allocate LCD COM Pins to LCD     // 01/26/2016
                                              //  function
      if(!lcdCOMPinHandle) {                                                      // 01/26/2016
          System_abort("Error initializing board LCD pins\n");                    // 01/26/2016
      }
      lcdSegPinHandle = PIN_open(&lcdSegPinState, lcdSegPinTableState);
                                              // allocate LCD Segment Pins to     // 01/27/2016
                                              // LCD function
      if(!lcdSegPinHandle) {                                                      // 01/27/2016
        System_abort("Error initializing board LCD pins\n");                      // 01/27/2016
      }

      /* SysMin will only print to the console when you call flush or exit */
      System_flush();

    /* Initialize task */
   TxTask_init(ledPinHandle); //FG: Disable to only measure LCD
    //MS Set LCD to OFF

    PIN_setPortOutputValue(lcdCOMPinHandle,  0x00000000);
                                          // push COM + Base_PWM values into  // 01/27/2016
                                          // port output
    PIN_setPortOutputValue(lcdSegPinHandle, 0x00000000);
                                          // push Segment Pin output values   // 01/28/2016
                                          // into port output
    PIN_setPortOutputEnable(lcdSegPinHandle, 0xFFFFFFFF);
                                          // switch Output buffers on/off     // 01/28/2016
                                          // according to desired Segment activity

    //Semaphore_post(semLCDHandle); //FG: Enable when TxTask_init(ledPinHandle) commented out

    /* Start BIOS */
    BIOS_start();

    return (0);
}

//FG 25.10.16
//Immediate M3 wake up event to allow recharge period algorithm to find an optimal recharge period

Void clkFxn(UArg arg0)
{
    static int counter = 0;
    if (++counter > 50)
    {
        clkHandle = Clock_handle(&clkStruct);
        Clock_stop(clkHandle);
    }
}
